"""
Gantt Chart
-----------------
This example shows how to make a simple gantt chart.
"""
# category: basic charts

import altair as alt
import pandas as pd

data = pd.DataFrame([
    {"task": "A","start": 1, "end": 3},
    {"task": "B","start": 3, "end": 8},
    {"task": "C","start": 8, "end": 10}
])

chart = alt.Chart(data).mark_bar().encode(
    x='start',
    x2='end',
    y='task',
)
